! $Id: omegalib.nw,v 1.118.4.2 2006/05/15 08:39:27 ohl Exp $
!
! Copyright (C) 2000-2002 by Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
! and others
!
! O'Mega is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by 
! the Free Software Foundation; either version 2, or (at your option)
! any later version.
!
! O'Mega is distributed in the hope that it will be useful, but
! WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
module omegavm95
  use omega_kinds
  use omega95
  ! use omega95_bispinors
  implicit none
  private
  public :: alloc
  public :: eval
  type, public :: ovm
     private
     complex(kind=omega_prec) :: amp
     type(momentum), dimension(:), pointer :: p
     complex(kind=omega_prec), dimension(:), pointer :: phi
     type(spinor), dimension(:), pointer :: psi
     type(conjspinor), dimension(:), pointer :: psibar
     ! type(bispinor), dimension(:), pointer :: chi
     type(vector), dimension(:), pointer :: v
  end type ovm
  type, public :: ovm_env
     private
     real(kind=omega_prec), dimension(:), pointer :: gr
     real(kind=omega_prec), dimension(:,:), pointer :: gr2
     complex(kind=omega_prec), dimension(:), pointer :: gc
     complex(kind=omega_prec), dimension(:,:), pointer :: gc2
  end type ovm_env
  integer, parameter, private :: MAX_RHS = 3
  type, public :: instr
     private
     integer :: code, sign, coupl, lhs
     integer, dimension(MAX_RHS) :: rhs
  end type instr
  integer, public, parameter :: OVM_LOAD_SCALAR = 1
  integer, public, parameter :: OVM_LOAD_U = 2
  integer, public, parameter :: OVM_LOAD_UBAR = 3
  integer, public, parameter :: OVM_LOAD_V = 4
  integer, public, parameter :: OVM_LOAD_VBAR = 5
  integer, public, parameter :: OVM_LOAD_VECTOR = 6
  integer, public, parameter :: OVM_ADD_MOMENTA = 10
  integer, public, parameter :: OVM_PROPAGATE_SCALAR = 11
  integer, public, parameter :: OVM_PROPAGATE_SPINOR = 12
  integer, public, parameter :: OVM_FUSE_VECTOR_PSIBAR_PSI = 21
  integer, public, parameter :: OVM_FUSE_PSI_VECTOR_PSI = 22
  integer, public, parameter :: OVM_FUSE_PSIBAR_PSIBAR_VECTOR = 23
contains
  subroutine alloc (vm, momenta, scalars, spinors, conjspinors, vectors)
    type(ovm), intent(inout) :: vm
    integer, intent(in) :: momenta, scalars, spinors, conjspinors, vectors
    allocate (vm%p(momenta))
    allocate (vm%phi(scalars))
    allocate (vm%psi(spinors))
    allocate (vm%psibar(conjspinors))
    allocate (vm%v(vectors))
  end subroutine alloc
  pure subroutine eval (vm, amp, env, amplitude, p, s)
    type(ovm), intent(inout) :: vm
    complex(kind=omega_prec), intent(out) :: amp
    type(ovm_env), intent(in) :: env
    type(instr), dimension(:), intent(in) :: amplitude
    real(kind=omega_prec), dimension(0:,:), intent(in) :: p
    integer, dimension(:), intent(in) :: s
    integer :: code, sign, coupl, lhs
    integer, dimension(MAX_RHS) :: rhs
    integer :: i, pc
    vm%p(1) = - p(:,1)
    vm%p(2) = - p(:,2)
    do i = 3, size (p, dim = 2)
       vm%p(i) = p(:,i)
    end do
    do pc = 1, size (amplitude)
       code = amplitude(pc)%code
       sign = amplitude(pc)%sign
       coupl = amplitude(pc)%coupl
       lhs = amplitude(pc)%lhs
       rhs = amplitude(pc)%rhs
       select case (code)
       case (OVM_LOAD_SCALAR)
          vm%phi(lhs) = 1
       case (OVM_LOAD_U)
          if (lhs <= 2) then
             vm%psi(lhs) = u (env%gr(coupl), - vm%p(rhs(1)), s(rhs(2)))
          else
             vm%psi(lhs) = u (env%gr(coupl), vm%p(rhs(1)), s(rhs(2)))
          end if
       case (OVM_LOAD_UBAR)
          if (lhs <= 2) then
             vm%psibar(lhs) = ubar (env%gr(coupl), - vm%p(rhs(1)), s(rhs(2)))
          else
             vm%psibar(lhs) = ubar (env%gr(coupl), vm%p(rhs(1)), s(rhs(2)))
          end if
       case (OVM_LOAD_V)
          if (lhs <= 2) then
             vm%psi(lhs) = v (env%gr(coupl), - vm%p(rhs(1)), s(rhs(2)))
          else
             vm%psi(lhs) = v (env%gr(coupl), vm%p(rhs(1)), s(rhs(2)))
          end if
       case (OVM_LOAD_VBAR)
          if (lhs <= 2) then
             vm%psibar(lhs) = vbar (env%gr(coupl), - vm%p(rhs(1)), s(rhs(2)))
          else
             vm%psibar(lhs) = vbar (env%gr(coupl), vm%p(rhs(1)), s(rhs(2)))
          end if
       case (OVM_LOAD_VECTOR)
          if (lhs <= 2) then
             vm%v(lhs) = eps (env%gr(coupl), - vm%p(rhs(1)), s(rhs(2)))
          else
             vm%v(lhs) = eps (env%gr(coupl), vm%p(rhs(1)), s(rhs(2)))
          end if
       case (OVM_ADD_MOMENTA)
          vm%p(lhs) = vm%p(rhs(1)) + vm%p(rhs(2))
       case (OVM_PROPAGATE_SCALAR)
          vm%phi(lhs) = pr_phi (vm%p(lhs),env%gr(rhs(1)),env%gr(rhs(2)),vm%phi(lhs))
       case (OVM_PROPAGATE_SPINOR)
          vm%psi(lhs) = pr_psi (vm%p(lhs),env%gr(rhs(1)),env%gr(rhs(2)),vm%psi(lhs))
       case (OVM_FUSE_VECTOR_PSIBAR_PSI)
          vm%v(lhs) = &
               v_ff (sign*env%gc(coupl), vm%psibar(rhs(1)), vm%psi(rhs(2)))
       case (OVM_FUSE_PSI_VECTOR_PSI)
          vm%psi(lhs) = &
               f_vf (sign*env%gc(coupl), vm%v(rhs(1)), vm%psi(rhs(2)))
       case (OVM_FUSE_PSIBAR_PSIBAR_VECTOR)
          vm%psibar(lhs) = &
               f_fv (sign*env%gc(coupl), vm%psibar(rhs(1)), vm%v(rhs(2)))
       end select
    end do
    amp = vm%amp
  end subroutine eval
end module omegavm95
